#include <netdb.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <pthread.h>
#include <stdio.h>
#include <string.h>

#include "response.h"
#include "scan.h"

extern pthread_mutex_t g_getservent;

static char *get_service_name(int port, char *proto)
{
	pthread_mutex_lock(&g_getservent);
	struct servent *servent = getservbyport(htons(port), proto);
	pthread_mutex_unlock(&g_getservent);
	if (!servent)
		return NULL;
	return strdup(servent->s_name);
}

void tcp_response(const struct tcphdr *tcphdr, const struct scan *data)
{
	const e_scantype type = data->type;
	if (type == SCAN_UDP) {
		dprintf(2, "ft_nmap: error: received a TCP responses for an UDP "
		           "scan\n");
		return;
	}
	if (data->responses->service == NULL)
		data->responses->service = get_service_name(data->port_start, "tcp");
	if (type == SCAN_SYN) {
		if (tcphdr->ack == 1 && tcphdr->syn == 1)
			data->responses->states[type] = OPENED;
		else if (tcphdr->ack == 1 && tcphdr->rst == 1)
			data->responses->states[type] = CLOSED;
	} else if (type == SCAN_ACK && tcphdr->rst == 1)
		data->responses->states[type] = UNFILTERED;
	else if (type == SCAN_NULL && tcphdr->rst == 1)
		data->responses->states[type] = CLOSED;
	else if (type == SCAN_FIN && tcphdr->rst == 1)
		data->responses->states[type] = CLOSED;
	else if (type == SCAN_XMAS && tcphdr->rst == 1)
		data->responses->states[type] = CLOSED;
}

void udp_response(const struct udphdr *udphdr, const struct scan *data)
{
	(void)udphdr;
	if (data->type != SCAN_UDP) {
		dprintf(2, "ft_nmap: error: received an UDP responses for a TCP "
		           "scan\n");
		return;
	}
	if (data->responses->service == NULL)
		data->responses->service = get_service_name(data->port_start, "udp");
	data->responses->states[SCAN_UDP] = OPENED;
}

void icmp_response(const struct icmphdr *icmphdr, const struct scan *data)
{
	const e_scantype type = data->type;

	data->responses->service = get_service_name(data->port_start, "udp");
	if (data->responses->service == NULL)
		data->responses->service = get_service_name(data->port_start, "tcp");
	if (type == SCAN_SYN && icmphdr->type == 3)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_ACK && icmphdr->type == 3)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_NULL && icmphdr->type == 3)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_FIN && icmphdr->type == 3)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_XMAS && icmphdr->type == 3)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_UDP && icmphdr->type == 3 && icmphdr->code == 3)
		data->responses->states[type] = CLOSED;
	else if (type == SCAN_UDP && icmphdr->type == 3 && icmphdr->code != 3)
		data->responses->states[type] = FILTERED;
}

void no_response(const struct scan *data)
{
	const e_scantype type = data->type;

	data->responses->service = get_service_name(data->port_start, "udp");
	if (data->responses->service == NULL)
		data->responses->service = get_service_name(data->port_start, "tcp");
	if (type == SCAN_SYN)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_ACK)
		data->responses->states[type] = FILTERED;
	else if (type == SCAN_NULL)
		data->responses->states[type] = OPENFILTERED;
	else if (type == SCAN_FIN)
		data->responses->states[type] = OPENFILTERED;
	else if (type == SCAN_XMAS)
		data->responses->states[type] = OPENFILTERED;
	else if (type == SCAN_UDP)
		data->responses->states[type] = OPENFILTERED;
}
